/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.accessor.reference;

import org.simantics.databoard.util.URIUtil;


/**
 * Reference by name to a record field.
 * Format: "n-<name>"
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class FieldNameReference extends AccessorReference {

	public String fieldName;
	
	public FieldNameReference(String fieldName) {
		this.fieldName = fieldName;
	}
	
	public FieldNameReference(String fieldName, AccessorReference child) {
		super(child);
		this.fieldName = fieldName;
	}
	
	public String getFieldName() {
		return fieldName;
	}
	
	@Override
	public String toString(boolean labelReference) {
		if (labelReference && (fieldName.equals("uv") || fieldName.equals("o") || fieldName.equals("v"))) labelReference = false;
		String encoded = URIUtil.encodeURI(fieldName);
		return labelReference ? encoded : "n-"+encoded; 
	}
	
	@Override
	public AccessorReference clone() {
		return new FieldNameReference(fieldName, childReference==null ? null : childReference.clone());
	}	

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof FieldNameReference == false) return false;
		FieldNameReference other = (FieldNameReference) obj;
		if (!other.fieldName.equals(fieldName)) return false;
		if (other.hasChildReference() != hasChildReference()) return false;
		if (hasChildReference() && !other.childReference.equals(childReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int hash = 4665456 + 37 * fieldName.hashCode();
		if (hasChildReference()) hash = 31*hash + childReference.hashCode();		
		return hash;
	}	
	
}

