/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.util.binary;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class ByteBufferReadable implements BinaryReadable, Seekable {
	
	ByteBuffer buf;
	
	public ByteBufferReadable(ByteBuffer buf) {		
		if (buf == null)
			throw new IllegalArgumentException("null");
		this.buf = buf;
	}

	public ByteBufferReadable(byte[] buf) {		
		if (buf == null)
			throw new IllegalArgumentException("null");
		this.buf = ByteBuffer.wrap(buf);		
	}

	
	@Override
	public byte get() {
		return buf.get();
	}

	@Override
	public void get(byte[] dst, int offset, int length) {
		buf.get(dst, offset, length);
	}

	@Override
	public void get(byte[] dst) {
		buf.get(dst);
	}

	@Override
	public void get(ByteBuffer buf) {		
		if (buf.hasArray()) {
			this.buf.get(buf.array(), buf.arrayOffset() + buf.position(), buf.remaining());
			buf.position(buf.capacity());
		} else {
			buf.put(buf);
		}
	}

	@Override
	public void get(ByteBuffer buf, int length) {
		if (buf.hasArray()) {
			this.buf.get(buf.array(), buf.arrayOffset() + buf.position(), length);
			buf.position(buf.position() + length);
		} else {
//			int len = Math.min( Math.min( buf.remaining(), this.buf.remaining() ), length);
			int len = length;
			int origLimit = this.buf.limit();
			try {
				this.buf.limit(this.buf.position()+len);
				buf.put(this.buf);
			} finally {
				this.buf.limit(origLimit);
			}
		}
	}

	@Override
	public double getDouble() {
		return buf.getDouble();
	}

	@Override
	public float getFloat() {
		return buf.getFloat();
	}

	@Override
	public int getInt() {
		return buf.getInt();
	}

	@Override
	public long getLong() {
		return buf.getLong();
	}

	@Override
	public short getShort() {
		return buf.getShort();
	}

	@Override
	public long length() {
		return buf.limit();
	}
	
	@Override
	public long position() {
		return buf.position();
	}
	
	public ByteOrder order() {
		return buf.order();
	}

	public void order(ByteOrder order) {
		buf.order(order);
	}

	public void position(int newPosition) throws IOException {
		buf.position(newPosition);
	}

	@Override
	public void position(long newPosition) throws IOException {
		if (newPosition>=Integer.MAX_VALUE || newPosition<0) throw new IllegalArgumentException("index out of range");
		buf.position((int) newPosition);
	}

	@Override
	public void skip(long bytes) throws IOException {
		long newPosition = bytes + position();
		position( newPosition );
	}
	
}
